#define GLOBAL
#include "set2d.h"
#undef  GLOBAL

/*****************************************************************************
*                         Simulation Programs                                *
*****************************************************************************/
int main()
{
  ini_bunch();
  out_bunch();

  printf("Now simulating ...\n");
  for(lap=1;lap<=tmx;lap++)
    {
#ifdef laser
      cal_Laser();
#endif
      dis_EpBp();
      cal_v();
      
      cal_p();
      cal_j();
      
      cal_B(qdt);
      cal_E(hdt); 
      cal_B(hdt);
      cal_E(hdt); 
      cal_B(qdt);

      out_bunch();
      if(lap % STEP == 0)
	printf("now process ... %3.0f %%\n",
	       (double)(lap)/(double)(tmx)*100);
    }
  printf("Complated \a\n");
  return 0;
}

void ini_bunch(void)
{
  printf("Now initialzation ...\n");

  ini_c();
  ini_p();
  ini_v();
  
#ifdef system_setup
  out_system_setup();
#endif

  printf("End initialzation \n");
}

void ini_c(void)
{
  lap = 0;
  
  vtx[0] = vtxe; vtx[1] = vtxi;
  vty[0] = vtye; vty[1] = vtyi;
  vtz[0] = vtze; vtz[1] = vtzi;

  vdx[0] = vdxe; vdx[1] = vdxi;
  vdy[0] = vdye; vdy[1] = vdyi;
  vdz[0] = vdze; vdz[1] = vdzi;

  m[0] = me; m[1] = mi;
  q[0] = qe; q[1] = qi;

  for(i=0;i<=1;i++)
    for(j=0;j<nmx;j++)
      x[i][j] = y[i][j] = 0.0;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      vx[i][j] = vy[i][j] = vz[i][j] = 0.0;

  for(i=0;i<=imx+1;i++)
    for(j=0;j<=jmx;j++)
      Ex[i][j] = jx[i][j] = 0.0;

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx+1;j++)
      Ey[i][j] = jy[i][j] = 0.0;

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      Bz[i][j] = 0.0;
}

void ini_v(void)
{
  double fvx,fvy,fvz;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
	do{
	  vx[i][j] = vy[i][j] = vz[i][j] = 0.0;
	  
	  for(k=0;k<12;k++)
	    {
	      vx[i][j] += rnd(1);
	      vy[i][j] += rnd(1);
	      vz[i][j] += rnd(1);
	    }
	  
	  vx[i][j] -= 6.0;
	  vy[i][j] -= 6.0;
	  vz[i][j] -= 6.0;
	  
	  vx[i][j] = vdx[i] + vtx[i]*vx[i][j];
	  vy[i][j] = vdy[i] + vty[i]*vy[i][j];
	  vz[i][j] = vdz[i] + vtz[i]*vz[i][j];
	  
	  fvx = fabs(vx[i][j]);
	  fvy = fabs(vy[i][j]);
	  fvz = fabs(vz[i][j]);
	  
	}while(fvx >= 1.0 || fvy >= 1.0 || fvz >= 1.0);
      }
}

void ini_p(void)
{
#ifdef gradient
  double psg = lg / 2.0;    /* area of gradient        */
  double psf = lp;          /* area of flat            */
  double pss = psg+psf;     /* area of plasma          */
  double Ng  = psg*nmx/pss; /* number of gradient area */
  double Nf  = psf*nmx/pss; /* number of flat area     */
  
  /* layer of gradient plasma */
  for(i=0;i<2;i++)
    for(j=0;j<Ng;j++)
      x[i][j] = grad(lg,dbp-lg);
  
  /* layer of flat plazma  */
  for(i=0;i<2;i++)
    for(j=Ng;j<nmx;j++)
      x[i][j] = rnd(lp) + dbp;

  np   = Nf/((lp/dx)*(double)(jmx));
#else
  /* all layer of plazma   */
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      x[i][j] = rnd(lp) + dbp;
  
  np = (double)(nmx)/((lp/dx)*(double)(jmx));
#endif
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      y[i][j] = rnd(Ly);
  
  norm = no/np;
}

/*****************************************************************************
*                         Simulation Functions                               *
*****************************************************************************/
void cal_p(void)
{
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
	hx[i][j] = x[i][j];
	hy[i][j] = y[i][j];

	hvx[i][j]= vx[i][j];
	
        x[i][j] += vx[i][j]*dt;
        y[i][j] += vy[i][j]*dt;
	
	hx[i][j] = (hx[i][j] + x[i][j])/2.0;
	hy[i][j] = (hy[i][j] + y[i][j])/2.0;

        /* Boundary conditions of particles */
	if( x[i][j] < 0.0){  x[i][j] *=-1.0;              vx[i][j] *= -1.0; }
        if( x[i][j] >= Lx){  x[i][j]  = 2.0*Lx- x[i][j];  vx[i][j] *= -1.0; }
	if(hx[i][j] < 0.0){ hx[i][j] *=-1.0;             hvx[i][j] *= -1.0; }
        if(hx[i][j] >= Lx){ hx[i][j]  = 2.0*Lx-hx[i][j]; hvx[i][j] *= -1.0; }

        if( y[i][j] < 0.0){  y[i][j] += Ly; }
        if( y[i][j] >= Ly){  y[i][j] -= Ly; }
	if(hy[i][j] < 0.0){ hy[i][j] += Ly; }
        if(hy[i][j] >= Ly){ hy[i][j] -= Ly; }
      }
}

void cal_v(void)
{
  double vxm,vxd,vxi;
  double vym,vyd,vyi;
  double vzm,vzd,vzi;  
  double bzg,qmt,Loz;

  /* by Buneman-Bris Method */
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      { 
        qmt = q[i]/m[i]*hdt;
        /* v- */
        Loz = 1.0/sqrt(1.0 - vav(vx[i][j],vy[i][j],vz[i][j]));
        vxm = Loz*vx[i][j] + qmt*Epx[i][j];
        vym = Loz*vy[i][j] + qmt*Epy[i][j];
        vzm = Loz*vz[i][j];
	
        /* v' */
        Loz = 1.0/sqrt(1.0 + vav(vxm,vym,vzm));
        bzg = qmt*Loz*Bpz[i][j];
        vxd = vxm + vym*bzg;
        vyd = vym - vxm*bzg;
        vzd = vzm;
	
        /* v+ */
	Loz = 2.0*bzg/(1.0 + (bzg*bzg));
        vxi = vxm + vyd*Loz + qmt*Epx[i][j];
        vyi = vym - vxd*Loz + qmt*Epy[i][j];
        vzi = vzm;
	
        /* vn */
        Loz = 1.0/sqrt(1.0 + vav(vxi,vyi,vzi));
	vx[i][j] = Loz*vxi;
        vy[i][j] = Loz*vyi;
        vz[i][j] = Loz*vzi;
      }
}

void cal_B(double ddt)
{
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmp;j++)
      tmp[i][j] = Bz[i][j];
  
  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      Bz[i][j] += ((Ex[i][j]-Ex[i][j-1])/dy-(Ey[i][j]-Ey[i-1][j])/dx)*ddt;

  /* Boundary conditions of absorption */
  for(j=1;j<=jmx;j++)
    {
      Bz[  0][j] = tmp[  1][j]+(dt_dx(ddt))*(Bz[  1][j]-tmp[  0][j]);
      Bz[imx][j] = tmp[imm][j]+(dt_dx(ddt))*(Bz[imm][j]-tmp[imx][j]);
    }
  
  /* Boundary conditions of magnetic field */
  for(i=0;i<=imp;i++)
    {
      Bz[i][  0] = Bz[i][jmx];
      Bz[i][jmp] = Bz[i][  1];
    }
}

void cal_E(double ddt)
{
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      Ex[i][j] += ((Bz[i][j+1]-Bz[i][j])/dy-jx[i][j]*norm)*ddt;
  
  /* Boundary conditions of Absorption */
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      tmp[i][j] = Ey[i][j];
  
  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      Ey[i][j] += ((-Bz[i+1][j]+Bz[i][j])/dx-jy[i][j]*norm)*ddt;
  
  /* Boundary conditions of Absorption */
  for(j=1;j<=jmx;j++)
    {
      Ey[  0][j] = tmp[  1][j]+(dt_dx(ddt))*(Ey[  1][j]-tmp[  0][j]);
      Ey[imx][j] = tmp[imm][j]+(dt_dx(ddt))*(Ey[imm][j]-tmp[imx][j]);
    }
  
  /* Boundary conditions of electric field */
  for(i=0;i<=imx;i++)
    {
      Ey[i][  0] = Ey[i][jmx];
      Ey[i][jmp] = Ey[i][  1];
    }
}

void dis_EpBp(void)
{
  double Ex1,Ex2,Ex3,Ex4;
  double Ey1,Ey2,Ey3,Ey4;
  double Bz1,Bz2,Bz3,Bz4;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cell(x[i][j],y[i][j],hdx,hdy);
        Bz1 = Bz[a  ][b  ]; 
	Bz2 = Bz[a+1][b  ];
        Bz3 = Bz[a+1][b+1]; 
	Bz4 = Bz[a  ][b+1];
	Bpz[i][j] = (s[2]*Bz1 + s[3]*Bz2 + s[0]*Bz3 + s[1]*Bz4)/ss;
	
        dis_cell(x[i][j],y[i][j],hdx,0.0);
        Ex1 = Ex[a  ][b  ]; 
	Ex2 = Ex[a+1][b  ];
        Ex3 = Ex[a+1][b+1]; 
	Ex4 = Ex[a  ][b+1];
        Epx[i][j] = (s[2]*Ex1 + s[3]*Ex2 + s[0]*Ex3 + s[1]*Ex4)/ss;

        dis_cell(x[i][j],y[i][j],0.0,hdy);
        Ey1 = Ey[a  ][b  ]; 
	Ey2 = Ey[a+1][b  ];
        Ey3 = Ey[a+1][b+1]; 
	Ey4 = Ey[a  ][b+1];
        Epy[i][j] = (s[2]*Ey1 + s[3]*Ey2 + s[0]*Ey3 + s[1]*Ey4)/ss;
      }
}

void dis_cell(double xx,double yy,double ddx,double ddy)
{
  double  lx, ly,_lx,_ly;

  a = (int)((xx + ddx)/dx);
  b = (int)((yy + ddy)/dy);
  
  lx = (xx + ddx)-(double)(a * dx);
  ly = (yy + ddy)-(double)(b * dy);
  
  _lx = dx - lx; 
  _ly = dy - ly; 

  s[0] =  lx *  ly; 
  s[1] = _lx *  ly;
  s[2] = _lx * _ly; 
  s[3] =  lx * _ly;
}

void cal_Laser(void)
{
  double t  = lap*dt;
  double yo = (jmx)/2.0;
  
  if(lap <= (int)(llt))
    {
      for(j=1;j<=jmx;j++)
	{
	  /* Gaussian profile */
	  Ey[0][j] = Eo*sin(t/tau)*sin(t/tau)*\
	    exp(-(j-yo)*dy*(j-yo)*dy/(w*w))*sin(-t);
	}
    }
}

void cal_j(void)
{
  for(i=0;i<=imx+1;i++)
    for(j=0;j<=jmx;j++)
      jx[i][j] = 0.0;

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx+1;j++)
      jy[i][j] = 0.0;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cell(hx[i][j],hy[i][j],hdx,0.0);
	
        jx[a  ][b  ] += q[i]*hvx[i][j]*(s[2]/ss);
        jx[a+1][b  ] += q[i]*hvx[i][j]*(s[3]/ss);
        jx[a+1][b+1] += q[i]*hvx[i][j]*(s[0]/ss);
        jx[a  ][b+1] += q[i]*hvx[i][j]*(s[1]/ss);
      }
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cell(hx[i][j],hy[i][j],0.0,hdy);
	
        jy[a  ][b  ] += q[i]*vy[i][j]*(s[2]/ss);
        jy[a+1][b  ] += q[i]*vy[i][j]*(s[3]/ss);
        jy[a+1][b+1] += q[i]*vy[i][j]*(s[0]/ss);
        jy[a  ][b+1] += q[i]*vy[i][j]*(s[1]/ss);
      }
  
  /* Boundary conditions of current density */
  for(i=0;i<=imp;i++)
    jx[i][jmx] = jx[i][0] = jx[i][jmx] + jx[i][0];
  
  for(i=0;i<=imx;i++)
    {
      jy[i][jmx] = jy[i][0] = jy[i][jmx] + jy[i][0];
      jy[i][jmp] = jy[i][1] = jy[i][jmp] + jy[i][1];
    }
}
