/*****************************************************************************
*                    Laser Plasma Interaction Simulator                      *
*                                    Version 1.0a                            *
*                                       2005.8.30                            *
*                          Copyright  Keita Ogawa                            * 
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#ifdef   GLOBAL
#define _GLOBAL
#else
#define _GLOBAL extern
#endif

/*****************************************************************************
*                            Simulation Parameter                            *
*****************************************************************************/
#define dx      0.3                        /* xζֹ             */
#define dy      0.3                        /* yζֹ             */
#define dt      0.1                        /* ֹ                    */

#define imx     200                        /* xΥ                 */
#define jmx     100                        /* yΥ                 */
#define nmx  200000                        /* Ķγҿ                      */
#define tmx    4000                        /* ׻(ƥå׿)          */

#define me      1.0                        /* ŻҼ                      */
#define mi   1837.0                        /*                     */
#define qe     -1.0                        /* ŻŲ                      */
#define qi      1.0                        /* Ų                    */

#define vtxe    0.04422895799              /* ŻҽǮ® vtxe (1keV)    */
#define vtye    0.04422895799              /* ŻҽǮ® vtye (1keV)    */
#define vtze    0.04422895799              /* ŻҽǮ® vtze (1keV)    */
#define vdxe    0.0                        /* Żҽɥե® vdxe     */
#define vdye    0.0                        /* Żҽɥե® vdye     */
#define vdze    0.0                        /* Żҽɥե® vdze     */

#define vtxi    0.0009229902706            /* Ǯ® vtxi (.8keV) */
#define vtyi    0.0009229902706            /* Ǯ® vtyi (.8keV) */
#define vtzi    0.0009229902706            /* Ǯ® vtzi (.8keV) */
#define vdxi    0.0                        /* ɥե® vdxi   */
#define vdyi    0.0                        /* ɥե® vdyi   */
#define vdzi    0.0                        /* ɥե® vdzi   */

#define Eo   2.866765509                   /* 졼ٿ Eo           */
                                           /*           I_19 : 2.866765509  */
                                           /*           I_20 : 9.065508526  */
#define tau  (1.77702978*40.0)             /* ѥ륹                      */
#define w    (5.927533309*10.)             /* ӡ                      */

#define no   (1.0)                         /* ׳̩                    */
#define lp   (5.927533309*6.0)             /* ץ饺ɤθ              */
#define lg   (5.927533309)                 /* ̩ٸۤθ                */
#define dbp  (5.927533309*2.0)             /* 饿åȤޤǤεΥ  */
#define um   (5.927533309)                 /* ʲ줿 1um              */ 

#define FBUF   50                          /* եХåե              */
#define BEGIN 0.0                          /* Ͽϻ(ƥå׿)      */
#define STEP  200                          /* Ͽֳ(ƥå׿)          */

/*****************************************************************************
*                         Useful Parameter definition                        *
*****************************************************************************/
#define hdx   (dx/2.0)                     /* half of dx                    */
#define hdy   (dy/2.0)                     /* half of dy                    */
#define hdt   (dt/2.0)                     /* half of dt                    */
#define qdt   (dt/4.0)                     /* quarter of dt                 */

#define Lx    (imx*dx)                     /* width  of simulation area     */
#define Ly    (jmx*dy)                     /* height of simulation area     */

#define tg    (imx*jmx)

#define imm   (imx-1)
#define imp   (imx+1)
#define jmm   (jmx-1)
#define jmp   (jmx+1)

#define ss    (dx*dy)
#define twp   (3.141592654*2.0)
#define llt   (3.141592654*tau/dt)         /* 졼ȼͻ(ƥå׿)  */

#ifdef gradient
#define tlp   (lg+lp)                      /* total layer plazma wall       */
#else
#define tlp   (lp)
#endif

/*****************************************************************************
*                          Environment of Simulation                         *
*****************************************************************************/
#define laser                              /* irradiation of laser          */
#define _gradient                          /* gradient at plazma wall       */
#define _DEBUG                             /* for DEBUG mode                */
#define system_setup                       /* output system setup file      */
#define project_density
#define project_laser

#define energy_total                       /* check of total energy         */
#define _energy_current                    /* output energy current         */
#define energy_spectrum                    /* output energy spectrum        */
#define energy_gain                        /* output energy gain            */

#define particle_phase                     /* output particle of phase      */
#define _particle_phase_pm3d               /* output phase for gnuplot      */
#define particle_density_pm3d              /* output density for gnuplot    */
#define particle_distribution              /* output distribution of phase  */

#define _field_current                     /* output current  field (jx,jy) */
#define _field_electric                    /* output electric field (Ex,Ey) */
#define field_magnetic                     /* output magnetic field (Bz)    */
#define _field_RMS                         /* output root mean square       */
#define field_magnetic_hourly
#define field_absorption

#define thermalflux                        /* output thermal flux           */

/*****************************************************************************
*                           Simulation Properties                            *
*****************************************************************************/
_GLOBAL double   m[2];                     /* mass   of particle            */
_GLOBAL double   q[2];                     /* charge of particle            */
_GLOBAL double vtx[2];                     /* thermal Velocity of particle  */
_GLOBAL double vty[2];                     /* thermal Velocity of particle  */
_GLOBAL double vtz[2];                     /* thermal Velocity of particle  */
_GLOBAL double vdx[2];                     /* drift velocity(x) of particle */
_GLOBAL double vdy[2];                     /* drift velocity(y) of particle */
_GLOBAL double vdz[2];                     /* drift velocity(z) of particle */

_GLOBAL double   x[2][nmx];                /* position (x) of particle      */
_GLOBAL double   y[2][nmx];                /* position (y) of particle      */
_GLOBAL double  hx[2][nmx];                /* position (x) of particle      */
_GLOBAL double  hy[2][nmx];                /* position (y) of particle      */
_GLOBAL double  vx[2][nmx];                /* velocity (x) of particle      */
_GLOBAL double  vy[2][nmx];                /* velocity (y) of particle      */
_GLOBAL double  vz[2][nmx];                /* velocity (z) of particle      */
_GLOBAL double hvx[2][nmx];                /* velocity (x) of particle      */
_GLOBAL double Epx[2][nmx];                /* bound electric field (x)      */
_GLOBAL double Epy[2][nmx];                /* bound electric field (y)      */
_GLOBAL double Bpz[2][nmx];                /* bound magnetic field (z)      */

_GLOBAL double  Ex[imx+2][jmx+1];          /* electric field (x)            */
_GLOBAL double  Ey[imx+1][jmx+2];          /* electric field (y)            */
_GLOBAL double  Bz[imx+2][jmx+2];          /* magnetic field (z)            */
_GLOBAL double  jx[imx+2][jmx+1];          /* current density (x)           */
_GLOBAL double  jy[imx+1][jmx+2];          /* current density (y)           */
_GLOBAL double tmp[imx+2][jmx+2];          /* temp                          */

#ifdef field_magnetic_hourly
_GLOBAL double Bhz[imx+1][jmx+1];          /* magnetic field (z)            */
#endif

_GLOBAL double s[4];                       /* areas for PIC-method          */ 
_GLOBAL double np;                         /* density (particle/cell)       */
_GLOBAL double norm;                       /* normalize density             */

_GLOBAL char fname[FBUF];                  /* filename variable             */
_GLOBAL int  i,j,k,lap;                    /* counter variable              */
_GLOBAL int  a,b;                          /* neighborhood (x,y)            */

/*****************************************************************************
*                         Function Declarations                              *
*****************************************************************************/
void ini_bunch                 (void);     /* initial particle & field      */
void ini_c                     (void);     /* initial constants             */
void ini_p                     (void);     /* initial paticles              */
void ini_v                     (void);     /* initial velocity              */

void cal_B                   (double);     /* calculate magnetic field      */
void cal_E                   (double);     /* calculate electric field      */
void cal_v                     (void);     /* calculate velocity (vx,vy)    */
void cal_p                     (void);     /* calculate particle (x,y)      */
void cal_j                     (void);     /* calculate current density     */
void cal_Laser                 (void);     /* irrandiate laser (y)          */

void out_bunch                 (void);     /* bunch of output functions     */
void out_system_setup          (void);     /* output system setup file      */
void out_particle_phase        (void);     /* output position and velocity  */
void out_particle_density_pm3d (void);     /* output pm3d of density        */
void out_particle_distribution (void);     /* output distribution of phase  */

void out_energy_spectrum       (void);     /* output energy spectrum        */
void out_energy_total          (void);     /* output total energy           */
void out_energy_gain           (void);
void out_field_electric        (void);     /* output electric field         */
void out_field_magnetic        (void);     /* output magnetic field         */
void out_field_RMS             (void);
void out_field_magnetic_hourly (void);
void out_field_absorption      (void);

void out_thermalflux           (void);     /* output thermal flux           */
void out_project_density       (void);
void out_project_laser         (void);

void dis_EpBp                  (void);     /* distribute Ep & Bp            */
void dis_cell(double,double,double,double);/* distribute cell               */

double rnd                   (double);     /* random function               */
double dt_dx                 (double);     /* operator for absorption       */
double grad           (double,double);     /* gradient                      */
double vav     (double,double,double);     /* vector abs value              */

/*****************************************************************************
*                    Laser Plasma Interaction Simulator                      *
*                                    Version 1.0a                            *
*                                       2005.8.30                            *
*                          Copyright  Keita Ogawa                            * 
*****************************************************************************/
